@@ -16,6 +16,7 @@ import string
 def _ (txt): return txt
 
 from rubber.util import *
+import rubber.modules
 
 #---------------------------------------
 
@@ -33,6 +34,7 @@ class Config (object):
 		self.latex = "latex"
 		self.cmdline = ["\\nonstopmode\\input{%s}"]
 		self.tex = "TeX"
+		self.loghead = re.compile("This is [0-9a-zA-Z-]*(TeX|Omega)")
 		self.paper = []
 
 	def find_input (self, name):
@@ -72,9 +74,10 @@ class Modules (Plugins):
 	package `rubber.modules'.
 	"""
 	def __init__ (self, env):
-		Plugins.__init__(self, "rubber.modules")
+		Plugins.__init__(self, rubber.modules.__path__)
 		self.env = env
 		self.objects = {}
+		self.commands = {}
 
 	def __getitem__ (self, name):
 		"""
@@ -82,12 +85,19 @@ class Modules (Plugins):
 		"""
 		return self.objects[name]
 
+	def has_key (self, name):
+		"""
+		Check if a given module is loaded.
+		"""
+		return self.objects.has_key(name)
+
 	def register (self, name, dict={}):
 		"""
 		Attempt to register a package with the specified name. If a module is
 		found, create an object from the module's class called `Module',
-		passing it the environment and `dict' as arguments. This dictionary
-		describes the command that caused the registration.
+		passing it the environment and `dict' as arguments, and execute all
+		delayed commands for this module. The dictionary describes the
+		command that caused the registration.
 		"""
 		r = Plugins.register(self, name)
 		if r == 0:
@@ -98,6 +108,12 @@ class Modules (Plugins):
 			return 2
 		mod = self.modules[name].Module(self.env, dict)
 		self.env.msg(2, _("module %s registered") % name)
+
+		if self.commands.has_key(name):
+			for (cmd,arg) in self.commands[name]:
+				mod.command(cmd, arg)
+			del self.commands[name]
+
 		self.objects[name] = mod
 		return 1
 
@@ -107,13 +123,26 @@ class Modules (Plugins):
 		"""
 		Plugins.clear(self)
 		self.objects = {}
+		self.commands = {}
+
+	def command (self, mod, cmd, arg):
+		"""
+		Send a command to a particular module. If this module is not loaded,
+		store the command so that it will be sent when the module is register.
+		"""
+		if self.objects.has_key(mod):
+			self.objects[mod].command(cmd, arg)
+		else:
+			if not self.commands.has_key(mod):
+				self.commands[mod] = []
+			self.commands[mod].append((cmd,arg))
 
 #---------------------------------------
 
 re_rerun = re.compile("LaTeX Warning:.*Rerun")
 re_file = re.compile("(\\((?P<file>[^ \n\t(){}]*)|\\))")
-re_badbox = re.compile("(Ov|Und)erfull \\\\[hv]box ")
-re_line = re.compile("l\\.(?P<line>[0-9]+)( (?P<text>.*))?$")
+re_badbox = re.compile(r"(Ov|Und)erfull \\[hv]box ")
+re_line = re.compile(r"l\.(?P<line>[0-9]+)( (?P<text>.*))?$")
 
 class LogCheck (object):
 	"""
@@ -124,6 +153,7 @@ class LogCheck (object):
 	def __init__ (self, env):
 		self.env = env
 		self.msg = env.msg
+		self.lines = None
 
 	def read (self, name):
 		"""
@@ -140,7 +170,7 @@ class LogCheck (object):
 		if not line:
 			file.close()
 			return 1
-		if line.find("This is " + self.env.conf.tex) == -1:
+		if not self.env.conf.loghead.match(line):
 			file.close()
 			return 1
 		self.lines = file.readlines()
@@ -217,7 +247,8 @@ class LogCheck (object):
 				if m:
 					parsing = 0
 					skipping = 1
-					self.msg.error(pos[-1], int(m.group("line")),
+					self.msg.error(
+						{ "file": pos[-1], "line": int(m.group("line")) },
 						error, m.group("text"))
 				elif line[0:3] == "***":
 					parsing = 0
@@ -246,7 +277,7 @@ class LogCheck (object):
 
 #---------------------------------------
 
-re_comment = re.compile("(?P<line>[^%]*)(%.*)?")
+re_comment = re.compile(r"(?P<line>([^\\%]|\\%|\\)*)(%.*)?")
 re_command = re.compile("[% ]*(rubber: *(?P<cmd>[^ ]*) *(?P<arg>.*))?.*")
 re_input = re.compile("\\\\input +(?P<arg>[^{} \n\\\\]+)")
 re_kpse = re.compile("kpathsea: Running (?P<cmd>[^ ]*).* (?P<arg>[^ ]*)$")
@@ -325,6 +356,7 @@ class Environment (Depend):
 		self.hooks = {
 			"input" : self.h_input,
 			"include" : self.h_include,
+			"includeonly": self.h_includeonly,
 			"usepackage" : self.h_usepackage,
 			"RequirePackage" : self.h_usepackage,
 			"documentclass" : self.h_documentclass,
@@ -338,6 +370,8 @@ class Environment (Depend):
 		}
 		self.update_seq()
 
+		self.include_only = {}
+
 		self.convert = Converter({}, self.modules)
 
 		# description of the building process:
@@ -345,6 +379,8 @@ class Environment (Depend):
 		self.source_building = None
 		self.final = self
 		self.watched_files = {}
+		self.onchange_md5 = {}
+		self.onchange_cmd = {}
 		self.removed_files = []
 
 		# state of the builder:
@@ -368,6 +404,7 @@ class Environment (Depend):
 			self.process(self.source())
 		except EndDocument:
 			pass
+		self.set_date()
 		self.msg(2, _("dependencies: %r") % self.sources.keys())
 
 	def do_process (self, file, path, dump=None):
@@ -476,6 +513,18 @@ class Environment (Depend):
 					dict['opt'] = args[1]
 				self.modules.register(args[0], dict)
 
+		elif cmd == "onchange":
+			args = string.split(arg, maxsplit=1)
+			if len(args) < 2:
+				self.msg.info(pos, _("two arguments required for command 'onchange'"))
+			else:
+				file = args[0]
+				self.onchange_cmd[file] = args[1]
+				if exists(file):
+					self.onchange_md5[file] = md5_file(file)
+				else:
+					self.onchange_md5[file] = None
+
 		elif cmd == "paper":
 			self.conf.paper.extend(string.split(arg))
 
@@ -503,12 +552,11 @@ class Environment (Depend):
 		else:
 			lst = string.split(cmd, ".", 1)
 			if len(lst) > 1:
-				if modules.has_key(lst[0]):
-					self.modules[lst[0]].command(lst[1], arg)
+				self.modules.command(lst[0], lst[1], arg)
 			else:
 				self.msg.info(pos, _("unknown directive '%s'") % cmd)
 
-	def process (self, path):
+	def process (self, path, loc={}):
 		"""
 		This method is called when an included file is processed. The argument
 		must be a valid file name.
@@ -520,7 +568,7 @@ class Environment (Depend):
 		self.msg(2, _("parsing %s") % path)
 		file = open(path)
 		if not self.sources.has_key(path):
-			self.sources[path] = DependLeaf([path], self.msg)
+			self.sources[path] = DependLeaf([path], self.msg, loc)
 		try:
 			try:
 				self.do_process(file, path)
@@ -530,7 +578,7 @@ class Environment (Depend):
 		except EndInput:
 			pass
 
-	def input_file (self, name):
+	def input_file (self, name, loc={}):
 		"""
 		Treat the given name as a source file to be read. If this source can
 		be the result of some conversion, then the conversion is performed,
@@ -539,19 +587,25 @@ class Environment (Depend):
 		its dependency node. The return value is (None,None) is the source
 		could neither be read nor built.
 		"""
+		if name.find("\\") >= 0 or name.find("#") >= 0:
+			return None, None
+
 		for path in self.conf.path:
 			pname = join(path, name)
 			dep = self.convert(pname, self)
 			if dep:
+				dep.loc = loc
 				self.sources[pname] = dep
 				return pname, dep
 			dep = self.convert(pname + ".tex", self)
 			if dep:
+				dep.loc = loc
 				self.sources[pname] = dep
 				return pname + ".tex", dep
+
 		file = self.conf.find_input(name)
 		if file:
-			self.process(file)
+			self.process(file, loc)
 			return file, self.sources[file]
 		else:
 			return None, None
@@ -586,7 +640,7 @@ class Environment (Depend):
 		if the included file is found.
 		"""
 		if dict["arg"]:
-			self.input_file(dict["arg"])
+			self.input_file(dict["arg"], dict)
 
 	def h_include (self, dict):
 		"""
@@ -594,12 +648,27 @@ class Environment (Depend):
 		source in a way very similar to \\input, except that LaTeX also
 		creates .aux files for them, so we have to notice this.
 		"""
-		if dict["arg"]:
-			file, _ = self.input_file(dict["arg"])
-			if file:
-				if file[-4:] == ".tex":
-					file = file[:-4]
-				self.removed_files.append(basename(file) + ".aux")
+		if not dict["arg"]:
+			return
+		if self.include_only and not self.include_only.has_key(dict["arg"]):
+			return
+		file, _ = self.input_file(dict["arg"], dict)
+		if file:
+			if file[-4:] == ".tex":
+				file = file[:-4]
+			self.removed_files.append(basename(file) + ".aux")
+
+	def h_includeonly (self, dict):
+		"""
+		Called when the macro \\includeonly is found, indicates the
+		comma-separated list of files that should be included, so that the
+		othe \\include are ignored.
+		"""
+		if not dict["arg"]:
+			return
+		self.include_only = {}
+		for name in dict["arg"].split(","):
+			self.include_only[name] = None
 
 	def h_documentclass (self, dict):
 		"""
@@ -753,6 +822,8 @@ class Environment (Depend):
 			self.aux_md5 = None
 		self.aux_md5_old = None
 
+		self.log.read(self.src_base + ".log")
+
 		self.must_compile = 0
 		self.must_compile = self.compile_needed()
 
@@ -760,10 +831,12 @@ class Environment (Depend):
 
 		for dep in self.sources.values():
 			if dep.make() == 0:
+				self.failed_dep = dep.failed_dep
 				return 1
 
 		for mod in self.modules.objects.values():
 			if mod.pre_compile():
+				self.failed_dep = mod
 				return 1
 		return 0
 		
@@ -774,6 +847,16 @@ class Environment (Depend):
 		each compilation of the main source. Returns true on failure.
 		"""
 		self.msg(2, _("running post-compilation scripts..."))
+
+		for file, md5 in self.onchange_md5.items():
+			if not exists(file):
+				continue
+			new = md5_file(file)
+			if md5 != new:
+				self.msg(0, _("running %s...") % self.onchange_cmd[file])
+				self.execute(["sh", "-c", self.onchange_cmd[file]])
+			self.onchange_md5[file] = new
+
 		for mod in self.modules.objects.values():
 			if mod.post_compile():
 				return 1
@@ -820,6 +903,10 @@ class Environment (Depend):
 		and 2 if something was done without failure.
 		"""
 		if self.pre_compile(): return 0
+
+		# If an error occurs after this point, it will be while LaTeXing.
+		self.failed_dep = self
+
 		if force or self.compile_needed():
 			self.must_compile = 0
 			if self.compile(): return 0
@@ -829,6 +916,9 @@ class Environment (Depend):
 				if self.compile(): return 0
 				if self.post_compile(): return 0
 
+		# Finally there was no error.
+		self.failed_dep = None
+
 		if self.something_done:
 			self.date = int(time.time())
 			return 2
@@ -848,8 +938,8 @@ class Environment (Depend):
 		if not exists(self.src_base + ".log"):
 			self.msg(3, _("the log file does not exist"))
 			return 1
-		if getmtime(self.src_base + ".log") < getmtime(self.source()):
-			self.msg(3, _("the source is younger than the log file"))
+		if getmtime(self.prods[0]) < getmtime(self.source()):
+			self.msg(3, _("the source is younger than the output file"))
 			return 1
 		if self.log.read(self.src_base + ".log"):
 			self.msg(3, _("the log file is not produced by %s") % self.conf.tex)
@@ -895,6 +985,9 @@ class Environment (Depend):
 				return 1
 		return 0
 
+	def show_errors (self):
+		self.log.show_errors()
+
 	###  utility methods
 
 	def watch_file (self, file):
@@ -951,6 +1044,14 @@ class Environment (Depend):
 		if env != {}:
 			self.msg(2, _("  with environment: %r") % env)
 
+		# We first look for the program to run so we can fail properly if the
+		# executable is not found.
+
+		progname = prog_available(prog[0])
+		if not progname:
+			self.msg(-1, _("%s not found") % prog[0])
+			return 1
+
 		penv = posix.environ.copy()
 		for (key,val) in env.items():
 			penv[key] = val
@@ -977,7 +1078,7 @@ class Environment (Depend):
 			os.dup2(f_err_w, sys.__stderr__.fileno())
 			if pwd:
 				os.chdir(pwd)
-			os.execvpe(prog[0], prog, penv)
+			os.execve(progname, prog, penv)
 
 		# The main process reads whatever is sent to the error stream and
 		# parses it for Kpathsea messages.
@@ -1054,13 +1155,13 @@ class Message (object):
 		"""
 		pass
 
-	def error (self, file, line, text, code):
+	def error (self, where, text, code=None):
 		"""
 		This method is called when the parsing of the log file found an error.
-		The arguments are, respectively, the name of the file and the line
-		number where the error occurred, the description of the error, and the
-		offending code (up to the error). The line number and the code may be
-		None, the file name and the text are required.
+		The arguments are, respectively, the positionr where the error
+		occurred (a dictionary that may contain entries 'file', 'line', etc),
+		the description of the error, and the offending code (up to the
+		error).
 		"""
 		pass
 
@@ -1120,3 +1221,9 @@ class Module (object):
 		"""
 		This is called when a directive for the module is found in the source.
 		"""
+
+	def show_errors (self):
+		"""
+		This is called if something has failed during an operation performed
+		by this module.
+		"""
