@@ -12,10 +12,22 @@ class Module (rubber.Module):
 	def __init__ (self, env, dict):
 		self.env = env
 		env.add_hook("verbatimtabinput", self.input)
-		env.add_hook("listinginput", self.input)
+		env.add_hook("listinginput", self.listinginput)
 
 	def input (self, dict):
 		if not dict["arg"]:
 			return 0
 		file = dict["arg"]
-		self.env.sources[file] = DependLeaf([file], self.env.msg)
+		if file.find("\\") < 0 and file.find("#") < 0:
+			self.env.sources[file] = DependLeaf([file], self.env.msg,
+					dict["pos"])
+
+	def listinginput (self, dict):
+		if not dict["arg"]:
+			return 0
+		file = get_next_arg(dict)
+		if not file:
+			return 0
+		if file.find("\\") < 0 and file.find("#") < 0:
+			self.env.sources[file] = DependLeaf([file], self.env.msg,
+					dict["pos"])
